------------------------------------------------------------------------------
-- SellableItemDrops - Logging your sellable item drops
------------------------------------------------------------------------------
-- Modules/GUI_history.lua - Interface/History Tab
--
-- Author: Expelliarm5s / September 2019 / All Rights Reserved
--
-- Version 0.4.15
------------------------------------------------------------------------------
local addonName, addon = ...
local GUI = addon:GetModule("GUI")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceGUI = LibStub("AceGUI-3.0")
local private = {}
------------------------------------------------------------------------------

GUI.guiTabs.texts["history"] = {
	title=L["History"], desc=L["Database"]}
	
GUI.guiTabs.order[#GUI.guiTabs.order + 1] = "history"

------------------------------------------------------------------------------
-- History Tab

-- create history tab
GUI.guiTabsSelect["history"] = function(container, group)
	GUI:DebugPrintf("Select Tab %s: %s", group, tostring(container))
	if not container then
		GUI:DebugPrintf("Select Tab: container is nil")
	end

	-- min. Size
	GUI.display.frame:SetMinResize(400, 400)

	-- Load History Data
	addon.Data:LoadDatabase()

	local tagList = {}
	for tag in pairs(addon.hisItems) do
		tagList[tag] = date(addon.timeFormat, tag)
		if (addon.hisInfo[tag].hisZone) then
			tagList[tag] = tagList[tag] .. " / " .. addon.hisInfo[tag].hisZone
		end
	end

	-- Filter: default filter is empty
	GUI.hisFilters = {}

	-- variants
	local flNotTheRealItemString = ""
	if (addon.db.global.useNotTheRealItemString) then
		flNotTheRealItemString = ITEM_QUALITY_COLORS[5].hex .. L[" (! = variant)"] .. FONT_COLOR_CODE_CLOSE
	end

	if (CanIMogIt) then
		flNotTheRealItemString = flNotTheRealItemString .. " " .. ITEM_QUALITY_COLORS[5].hex .. L[" (TM = New Transmog)"] .. FONT_COLOR_CODE_CLOSE
	end

	-- session selection group
	local grp1 = AceGUI:Create("SIDInlineGroup")
	grp1:SetLayout("Flow")
	if grp1.SetTitle then
		grp1:SetTitle(L["Session History Selection"])
	end
	grp1:SetFullWidth(true)

	local grpWidth = 1
	local grpColNum = 5.01
	local grpRelWidth = grpWidth / grpColNum

	-- session selection group - line 1
	GUI.lblHis = {}
	GUI.lblHis.startTime = GUI.AddButton("Label", L["Session"], "-")
	GUI.lblHis.startTime:SetRelativeWidth(0.45)
	grp1:AddChild(GUI.lblHis.startTime)

	GUI.lblHis.duration = GUI.AddButton("Label", L["Duration"], "-")
	GUI.lblHis.duration:SetRelativeWidth(0.14)
	grp1:AddChild(GUI.lblHis.duration)

	GUI.lblHis.liv = GUI.AddButton("Label", L["LIV"], "-")
	GUI.lblHis.liv:SetRelativeWidth(0.20)
	grp1:AddChild(GUI.lblHis.liv)

	GUI.lblHis.source = GUI.AddButton("Label", L["Price Source"], "-")
	GUI.lblHis.source:SetRelativeWidth(0.18)
	grp1:AddChild(GUI.lblHis.source)

	-- session selection group - line 2
	GUI.infoHis = {}
	GUI.infoHis.selection = GUI.AddButton("Dropdown", tagList[addon.hisCTag] or L["no sessions saved"], "-")
	GUI.infoHis.selection:SetRelativeWidth(0.45)
	GUI.infoHis.selection:SetList(tagList)
	GUI.infoHis.selection:SetCallback("OnValueChanged",
		function(_, _, value)
			GUI:DebugPrintf("History Selection, value=%s", value)
			addon.hisCTag = value
			addon.Data:UpdateDatabaseTag(addon.hisCTag)
			GUI.gotNIL = nil
			container:Reload()
		end
	)
	grp1:AddChild(GUI.infoHis.selection)

	GUI.infoHis.duration = GUI.AddButton("Label", "-", "-")
	GUI.infoHis.duration:SetRelativeWidth(0.14)
	grp1:AddChild(GUI.infoHis.duration)

	GUI.infoHis.liv = GUI.AddButton("Label", "-", "-")
	GUI.infoHis.liv:SetRelativeWidth(0.20)
	GUI.labelFont, GUI.labelFontSize = GUI.infoHis.liv.label:GetFont()
	GUI.infoHis.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize + 4)
	grp1:AddChild(GUI.infoHis.liv)

	GUI.infoHis.source = GUI.AddButton("Label", "-", "-")
	GUI.infoHis.source:SetRelativeWidth(0.18)
	grp1:AddChild(GUI.infoHis.source)

	container:AddChild(grp1)

	-- session history data group
	local grp2 = AceGUI:Create("SIDInlineGroup")
	grp2:SetLayout("Flow")
	if grp2.SetTitle then
		if addon.db.global.stackedView then
			grp2:SetTitle(L["Session History Data"] .. L[" (stacked view)"])
		else
			grp2:SetTitle(L["Session History Data"])
		end
	end
	grp2:SetFullWidth(true)
	grp2:SetFullHeight(true)

	local grp3 = AceGUI:Create("SimpleGroup")
	grp3:SetLayout("Flow")
	grp3:SetFullWidth(true)

	grpWidth = 1
	grpColNum = 5.01
	if not addon.db.global.shPlayer then
		grpColNum = grpColNum - 1
	end
	if not addon.db.global.shZone then
		grpColNum = grpColNum - 1
	end

	grpRelWidth = grpWidth / grpColNum

	-- session history data group - line 1
	local i = GUI.AddButton("Label", L["Search"], "-")
	i:SetRelativeWidth(grpRelWidth)
	grp3:AddChild(i)

	i = GUI.AddButton("Label", L["Rarity"], "-")
	i:SetRelativeWidth(grpRelWidth)
	grp3:AddChild(i)

	if addon.db.global.shPlayer then
		i = GUI.AddButton("Label", L["Player"], "-")
		i:SetRelativeWidth(grpRelWidth)
		grp3:AddChild(i)
	end

	if addon.db.global.shZone then
		i = GUI.AddButton("Label", L["Zone"], "-")
		i:SetRelativeWidth(grpRelWidth)
		grp3:AddChild(i)
	end

	i = GUI.AddButton("Label", "", "-")
	i:SetRelativeWidth(grpRelWidth)
	grp3:AddChild(i)
	grp2:AddChild(grp3)

	grp3 = AceGUI:Create("SimpleGroup")
	grp3:SetLayout("Flow")
	grp3:SetFullWidth(true)

	-- session history data group - line 2
	i = GUI.AddButton("SIDEditBox", "", "-")
	i:SetRelativeWidth(grpRelWidth)
	i.button:SetScript("OnClick", function(f) 
			local editbox = f.obj.editbox
			editbox:SetText("")
			editbox:SetCursorPosition(0)
			editbox:ClearFocus()
			f.obj:DisableButton(true)
			f.obj:Fire("OnTextChanged", "")
		end)
	i.button:SetText(L["CLR"])
	i:SetCallback("OnTextChanged",
		function(f, _, value)
			value = value:trim()
			if value == "" then
				GUI.hisFilters.name = nil
			else
				GUI.hisFilters.name = value
				f:DisableButton(false)
			end
			GUI.dataHis:SetContent(private.GetHistoryItemData(GUI.hisFilters))
			private.GetHistoryInfoData(GUI.hisFilters)
		end
	)
	grp3:AddChild(i)

	i = GUI.AddButton("Dropdown", L["Rarity"], "-")
	i:SetRelativeWidth(grpRelWidth)
	i:SetList(GUI.rarityList)
	i:SetCallback("OnValueChanged",
		function(_, _, key)
			if key > 0 then
				GUI.hisFilters.rarity = key
			else
				GUI.hisFilters.rarity = nil
			end
			GUI.dataHis:SetContent(private.GetHistoryItemData(GUI.hisFilters))
			private.GetHistoryInfoData(GUI.hisFilters)
		end
	)
	grp3:AddChild(i)

	if addon.db.global.shPlayer then
		i = GUI.AddButton("Dropdown", L["Player"], "-")
		i:SetRelativeWidth(grpRelWidth)
		i:SetList(GUI.playerListFromItems(addon.hisItems[addon.hisCTag]))
		local value = "all"
		i:SetText(i.list[value])
		i.value = value
		i:SetCallback("OnValueChanged",
			function(_, _, value)
				if value == "all" then
					GUI.hisFilters.player = nil
				else
					GUI.hisFilters.player = value
				end
				GUI.dataHis:SetContent(private.GetHistoryItemData(GUI.hisFilters))
				private.GetHistoryInfoData(GUI.hisFilters)
			end
		)
		grp3:AddChild(i)
	end

	if addon.db.global.shZone then
		i = GUI.AddButton("Dropdown", L["Zone"], "-")
		i:SetRelativeWidth(grpRelWidth)
		i:SetList(GUI.zoneListFromItems(addon.hisItems[addon.hisCTag]))
		local value = "all"
		i:SetText(i.list[value])
		i.value = value
		i:SetCallback("OnValueChanged",
			function(_, _, value)
				if value == "all" then
					GUI.hisFilters.zone = nil
				else
					GUI.hisFilters.zone = value
				end
				GUI.dataHis:SetContent(private.GetHistoryItemData(GUI.hisFilters))
				private.GetHistoryInfoData(GUI.hisFilters)
			end
		)
		grp3:AddChild(i)
	end

	i = GUI.AddButton(L["Delete"], L["Delete the displayed history session data."])
	i:SetRelativeWidth(grpRelWidth)
	i:SetCallback("OnClick",
		function()
			if (addon.db.global.nwSound ~= "") then
				addon.Options.PlaySound()
				GUI:DebugPrintf()
			end
			addon.Data:WipeHistoryTag(addon.hisCTag)
			addon.hisCTag = 0
			addon.Data:ResetCurrentTag(true)
			addon.Data:UpdateDatabaseTag(addon.hisCTag)
			container:Reload()
		end
	)
	grp3:AddChild(i)

	grp2:AddChild(grp3)

	-- session history data items
	local grp4 = AceGUI:Create("SimpleGroup")
	grp4:SetLayout("Fill")
	grp4:SetFullWidth(true)
	grp4:SetFullHeight(true)

	local titles = {}
	titles = {
		{text=L["Item"] .. flNotTheRealItemString, relWidth=GUI.TITLEWIDTH_ITEM},
		{text=L["Item Value"], relWidth=-GUI.TITLEWIDTH_VALUE},
		{text=L["Qty"], relWidth=-GUI.TITLEWIDTH_QTY},
		{text=L["Player"], relWidth=GUI.TITLEWIDTH_PLAYER},
		{text=L["Zone"], relWidth=GUI.TITLEWIDTH_ZONE},
		{text=L["Time"], relWidth=GUI.TITLEWIDTH_TIME},
	}
	local colSpan = 0
	local colSpan2 = 6
	if (not addon.db.global.shPlayer) then
		colSpan = colSpan + GUI.TITLEWIDTH_PLAYER - GUI.TITLEWIDTH_GAP
		titles[4] = {text="", relWidth=GUI.TITLEWIDTH_GAP}
		colSpan2 = colSpan2 - 1
	end
	if (not addon.db.global.shZone) then
		colSpan = colSpan + GUI.TITLEWIDTH_ZONE - GUI.TITLEWIDTH_GAP
		titles[5] = {text="", relWidth=GUI.TITLEWIDTH_GAP}
		colSpan2 = colSpan2 - 1
	end
	titles[1].relWidth = GUI.TITLEWIDTH_ITEM  + colSpan / colSpan2
	titles[2].relWidth = -GUI.TITLEWIDTH_VALUE - colSpan / colSpan2
	titles[3].relWidth = -GUI.TITLEWIDTH_QTY - colSpan / colSpan2
	if titles[4].text ~= "" then
		titles[4].relWidth = GUI.TITLEWIDTH_PLAYER + colSpan / colSpan2
	end
	if titles[5].text ~= "" then
		titles[5].relWidth = GUI.TITLEWIDTH_ZONE + colSpan / colSpan2
	end
	titles[6].relWidth = GUI.TITLEWIDTH_TIME + colSpan / colSpan2

	container.titleStatusText = {
		L["Leftclick to sort this column / Rightclick to show all columns"],
		L["Leftclick to sort this column"],
		L["Leftclick to sort this column"],
		L["Leftclick to sort and Rightclick to hide Player column"],
		L["Leftclick to sort and RightClick to hide Zone column"],
		L["Leftclick to sort this column"],
		}

	container.statusText = L["Alt-Rightclick to delete the item"]

	local eventHandlers = {
		OnEnter = GUI.DataFrame_OnEnter,
		OnLeave = GUI.DataFrame_OnLeave,
		OnClick = function(btn, data, button)
			if not data then return end
			-- for item debugging
			if button == "LeftButton" and IsShiftKeyDown() then
				GUI:PrintItemStrings(data.link)
			end
			-- for item debugging
			if button == "RightButton" and IsRightShiftKeyDown() then
				if testitem then
					local i = addon.Item:GetItemID(data.link)
					if i then
						testitem(data.link,i,1)
						addon.Data:InsertItem(data.link, 1, addon.Item:GetItemQuality(data.link,data.link))
						addon.Data:UpdateSession()
						addon.GUI:UpdateSessionData(true)
					end
				end
			end
			if button == "RightButton" and not IsAltKeyDown() then
				if type(data) == "number" then
					if data == 1 then
						addon.db.global.shPlayer = true
						addon.db.global.shZone = true
						container:Reload()
					end
					if data == 4 then
						addon.db.global.shPlayer = false
						container:Reload()
					end
					if data == 5 then
						addon.db.global.shZone = false
						container:Reload()
					end
				end
			end
			-- purge the line/item
			if button == "RightButton" and IsAltKeyDown() then
				if (addon.hisCTag > 0) then
					local qty = addon.Data:PurgeItem(data, addon.hisItems[addon.hisCTag])
					if (qty > 0) then
						addon.Data:SaveDatabaseTag(addon.hisCTag)
						addon.Data:LoadDatabaseTag(addon.hisCTag)
						container:Reload()
						addon:Printf(L["%sx %s purged."], qty, data.link)
					end
				end
			end
		end,
	}

	GUI.dataHis = AceGUI:Create("SIDData")
	GUI.dataHis:DoRedraw(false)
	GUI.dataHis:SetEventHandlers(eventHandlers)
	GUI.dataHis:SetTitles(titles)
	GUI.dataHis:SetSortTitle(-6)
	grp4:AddChild(GUI.dataHis)

	grp2:AddChild(grp4)
	container:AddChild(grp2)

	addon.Item:GotNILReset()
	GUI.dataHis:SetContent(private.GetHistoryItemData(GUI.hisFilters))
	GUI.dataHis:DoRedraw(true)

	private.GetHistoryInfoData(GUI.hisFilters)

	if addon.Item:GotNIL() then
		GUI.gotNIL = GUI.gotNIL or 5
		if GUI.gotNIL > 0 then
			GUI.gotNIL = GUI.gotNIL - 1
			addon.Data.secToUpdate = 1
			GUI:DebugPrintf("Got nil for GetItemInfo(), try reload #%s in %s sec.", tostring(GUI.gotNIL), tostring(addon.Data.secToUpdate))
		end
	else
		GUI.gotNIL = nil
	end
	
	return container
end

-- prepare/update history item data
function private.GetHistoryItemData(lfilters)
	GUI:DebugPrintf("GetHistoryItemData()")
	if GUI.view ~= "history" or not GUI.display then
		GUI:DebugPrintf("... nothing done.")
		return
	end

	if (addon.hisCTag > 0) then
		GUI:DebugPrintf("tag = %s", addon.hisCTag)
		return GUI.GetItemData(addon.hisItems[addon.hisCTag], lfilters)
	else
		GUI:DebugPrintf("no tag active!")
		return GUI.GetItemData({}, lfilters)
	end
end

-- prepare/update history info data
function private.GetHistoryInfoData(filters)
	GUI:DebugPrintf("GetHistoryInfoData()")
	if GUI.view ~= "history" or not GUI.display then
		GUI:DebugPrintf("... nothing done.")
		return
	end

	if (addon.hisCTag > 0) then
		if (addon.hisInfo[addon.hisCTag].hisMVSource ~= addon.db.global.mvSource) then
			GUI:DebugPrintf("mvSource changed, update data...")
			addon.Data:UpdateDatabaseTag(addon.hisCTag)
		end
	end

	local mvFilter = 0
	local nwFilter = 0
	local itsSum = 0
	local itsFilter = 0
	local itSum = 0
	local itFilter = 0
	local isFiltered = false

	if (not filters.name and not filters.rarity and not filters.player and not filters.time and not filters.zone) then
		GUI:DebugPrintf("... with no filters")
	else
		if (addon.hisCTag > 0) then
		GUI:DebugPrintf("... with filters")
		isFiltered = true
		itStart = 0
		itEnd = 0
		for link, data in pairs(addon.hisItems[addon.hisCTag]) do
			if #data.drops > 0 then
				itsSum = itsSum + 1
			end
			if #data.drops > 0 and not GUI.IsItemFiltered(link, data, filters) then
				itsFilter = itsFilter + 1
				for _, record in ipairs(data.drops) do
					itSum = itSum + record.quantity
					if not GUI.IsRecordFiltered(record, filters) then
						if (itStart == 0) then
							itStart = record.time
						end
						if (record.time < itStart) then
							itStart = record.time
						end
						if (record.time > itEnd) then
							itEnd = record.time
						end
						itFilter = itFilter + record.quantity
						local marketValue = addon.Data:GetItemValue(record.link) or 0
						mvFilter = mvFilter + (marketValue * record.quantity)
					end
				end
			end
		end
		end
	end

	if (isFiltered or (itsSum ~= itsFilter) or (itSum ~= itFilter)) then
		if (GUI.infoHis) then
			-- Duration
			GUI.lblHis.duration:SetText(L["Duration"])
			GUI.infoHis.duration:SetText(" " .. ITEM_QUALITY_COLORS[6].hex ..
				SecondsToTime(itEnd - itStart)
				.. FONT_COLOR_CODE_CLOSE)
			-- LIV / Gold
			GUI.lblHis.liv:SetText(L["LIV"])
			GUI.infoHis.liv:SetText(GUI:FromCopper(mvFilter))
			-- Price Source
			local mvSourcesList = addon.Item:GetPriceSources() or {}
			if mvSourcesList[addon.db.global.mvSource] then
				GUI.infoHis.source:SetText(ITEM_QUALITY_COLORS[6].hex ..
					addon.db.global.mvSource
					.. FONT_COLOR_CODE_CLOSE)
			else
				GUI.infoHis.source:SetText(ITEM_QUALITY_COLORS[0].hex ..
					"(" .. addon.db.global.mvSource .. ")"
					.. FONT_COLOR_CODE_CLOSE)
			end
		end
	else
		if (GUI.infoHis) then
			-- Duration
			GUI.lblHis.duration:SetText(L["Duration"])
			if (addon.hisCTag > 0) then
				GUI.infoHis.duration:SetText(" " .. ITEM_QUALITY_COLORS[6].hex ..
					SecondsToTime(addon.hisInfo[addon.hisCTag].hisEnd - addon.hisInfo[addon.hisCTag].hisTime)
					.. FONT_COLOR_CODE_CLOSE)
			end
			-- Looted Item Value / Gold
			GUI.lblHis.liv:SetText(L["LIV"])
			if (addon.hisCTag > 0) then
				GUI.infoHis.liv:SetText(GUI:FromCopper(addon.hisInfo[addon.hisCTag].hisMV))
			end
			-- Price Source
			local mvSourcesList = addon.Item:GetPriceSources() or {}
			if mvSourcesList[addon.db.global.mvSource] then
				GUI.infoHis.source:SetText(ITEM_QUALITY_COLORS[6].hex ..
					addon.db.global.mvSource
					.. FONT_COLOR_CODE_CLOSE)
			else
				GUI.infoHis.source:SetText(ITEM_QUALITY_COLORS[0].hex ..
					"(" .. addon.db.global.mvSource .. ")"
					.. FONT_COLOR_CODE_CLOSE)
			end
		end
	end
end

function GUI:UpdateHistoryData(sesItemsUpdated)
	if GUI.view == "history" then
		GUI:DebugPrintf("Adapt History")
		if GUI.labelFont and GUI.labelFontSize and GUI.container then
			if GUI.adapt.width and GUI.adapt.width >= 600 then
				GUI.infoHis.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize + 4)
				if GUI.TITLEWIDTH_ITEM ~= 0.31 then
					if GUI.adapt.shPlayer ~= nil then
						addon.db.global.shPlayer = GUI.adapt.shPlayer
						GUI.adapt.shPlayer = nil
					end
					if GUI.adapt.shZone ~= nil then
						addon.db.global.shZone = GUI.adapt.shZone
						GUI.adapt.shZone = nil
					end
					GUI.TITLEWIDTH_ITEM = 0.31
					GUI.TITLEWIDTH_VALUE = 0.17
					GUI.TITLEWIDTH_QTY = 0.04
					GUI.TITLEWIDTH_PLAYER = 0.10
					GUI.TITLEWIDTH_ZONE = 0.20
					GUI.TITLEWIDTH_TIME = 0.18
					-- GUI:Printf("History Reload >= 600")
					GUI.container:Reload()
				end
			elseif GUI.adapt.width and GUI.adapt.width >= 500 then
				GUI.infoHis.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize + 2)
				if GUI.TITLEWIDTH_ITEM ~= 0.35 then
					if GUI.adapt.shPlayer == nil then
						GUI.adapt.shPlayer = addon.db.global.shPlayer
						addon.db.global.shPlayer = false
					end
					if GUI.adapt.shZone ~= nil then
						addon.db.global.shZone = GUI.adapt.shZone
						GUI.adapt.shZone = nil
					end
					GUI.TITLEWIDTH_ITEM = 0.35
					GUI.TITLEWIDTH_VALUE = 0.21
					GUI.TITLEWIDTH_QTY = 0.04
					GUI.TITLEWIDTH_PLAYER = 0.10
					GUI.TITLEWIDTH_ZONE = 0.17
					GUI.TITLEWIDTH_TIME = 0.13
					-- GUI:Printf("History Reload >= 500")
					GUI.container:Reload()
				end
			else
				GUI.infoHis.liv.label:SetFont(GUI.labelFont, GUI.labelFontSize)
				if GUI.TITLEWIDTH_ITEM ~= 0.40 then
					if GUI.adapt.shPlayer == nil then
						GUI.adapt.shPlayer = addon.db.global.shPlayer
						addon.db.global.shPlayer = false
					end
					if GUI.adapt.shZone == nil then
						GUI.adapt.shZone = addon.db.global.shZone
						addon.db.global.shZone = false
					end
					GUI.TITLEWIDTH_ITEM = 0.40
					GUI.TITLEWIDTH_VALUE = 0.22
					GUI.TITLEWIDTH_QTY = 0.04
					GUI.TITLEWIDTH_PLAYER = 0.10
					GUI.TITLEWIDTH_ZONE = 0.13
					GUI.TITLEWIDTH_TIME = 0.11
					-- GUI:Printf("History Reload")
					GUI.container:Reload()
				end
			end
		end
	end
end

-- EOF

